import datetime

from sqlalchemy import Column, Integer, DateTime

from command_system import RollbotResponse, RollbotFailure, as_plugin, ModelBase


class DnDSession(ModelBase):
    __tablename__ = "dnd_session"
    id = Column(Integer, primary_key=True)
    session_time = Column(DateTime)


WEEKDAYS = {
    "monday": 0,
    "tuesday": 1,
    "wednesday": 2,
    "thursday": 3,
    "friday": 4,
    "saturday": 5,
    "sunday": 6
}


def parse_datetime(args):
    try:
        day_date = next(args)
    except StopIteration:
        print(1)
        return  # must have at least one more argument

    day_date = day_date.lower()
    if day_date in WEEKDAYS:
        today = datetime.date.today()
        day_diff = WEEKDAYS[day_date] - today.weekday()
        next_day = today + datetime.timedelta((day_diff) % 7)
        month = next_day.month
        day = next_day.day
        year = next_day.year
    else:
        date_parts = day_date.split("/")
        if len(date_parts) == 3:
            month = date_parts[0]
            day = date_parts[1]
            year = date_parts[2]
        elif len(date_parts) == 2:
            month = date_parts[0]
            day = date_parts[1]
            year = datetime.date.today().year
        else:
            print(2)
            return  # invalid format

    hour = 19
    minute = 45

    try:
        time = next(args)
    except StopIteration:
        return datetime.datetime(
            int(year), int(month), int(day),
            int(hour), int(minute)
        )

    time_parts = time.split(":")
    hour = time_parts[0]
    if len(time_parts) > 1:
        minute = time_parts[1]

    try:
        ampm = next(args)
    except StopIteration:
        return datetime.datetime(
            int(year), int(month), int(day),
            int(hour), int(minute)
        )

    if ampm.lower() == "pm":
        hour = int(hour) + 12

    return datetime.datetime(
        int(year), int(month), int(day),
        int(hour), int(minute)
    )



def get_upcoming(db):
    return db.query(DnDSession).order_by(DnDSession.session_time).first()


def get_fmt_next(db):
    dt = get_upcoming(db).session_time
    if dt is None or dt < datetime.datetime.now():
        return "No upcoming session found."
    else:
        return "Next DnD session scheduled for {}/{}/{} at {}:{:02d}.".format(
            dt.month, dt.day, dt.year, dt.hour, dt.minute)


def session_next(db, message, args):
    dt = parse_datetime(args)
    if dt is None:
        return RollbotResponse(
            message,
            failure=RollbotFailure.INVALID_ARGUMENTS,
            debugging={"explain": "Could not process argument as a date/time."}
        )

    db.add(DnDSession(session_time=dt))

    return RollbotResponse(message, txt=get_fmt_next(db))


def session_worst(db, message, args):
    return RollbotResponse(message, txt="TODO")


def session_view(db, message, args):
    return RollbotResponse(message, txt="TODO")


def session_late(db, message, args):
    return RollbotResponse(message, txt="TODO")


def session_cancel(db, message, args):
    return RollbotResponse(message, txt="TODO")


SUBCOMMANDS = dict(
    next=session_next,
    worst=session_worst,
    view=session_view,
    late=session_late,
    cancel=session_cancel
)

NEEDS_ADMIN = ("next", "cancel")


@as_plugin
def session(db, message):
    args = message.args()

    try:
        subcommand = next(args)
    except StopIteration:
        return RollbotResponse(message, failure=RollbotFailure.MISSING_SUBCOMMAND)

    if subcommand not in SUBCOMMANDS:
        return RollbotResponse(
            message,
            failure=RollbotFailure.INVALID_SUBCOMMAND,
            debugging={
                "explain": f"Must be one of: {', '.join(repr(s) for s in SUBCOMMANDS)}."
            }
        )

    if subcommand in NEEDS_ADMIN and not message.from_admin:
        return RollbotResponse(message, failure=RollbotFailure.PERMISSIONS)

    return SUBCOMMANDS[subcommand](db, message, args)
